ASI_GENERIC_PROPERTY_DATA_COLLECTION = 'Asi.Soa.Core.DataContracts.GenericPropertyDataCollection, Asi.Contracts'
ASI_GENERIC_PROPERTY_DATA = 'Asi.Soa.Core.DataContracts.GenericPropertyData, Asi.Contracts'
ASI_GENERIC_PROPERTY_DATA_TYPES = (
    (str, None),
    (float, 'System.Decimal'),
    (int, 'System.Int32'),
    (bool, 'System.Boolean'),
)


def generic_property_data(name, value):
    data = {
        '$type': ASI_GENERIC_PROPERTY_DATA,
        'Name': name
    }
    for instance_type, asi_type_label in ASI_GENERIC_PROPERTY_DATA_TYPES:
        if isinstance(value, instance_type):
            if asi_type_label is None:
                data['Value'] = value
            else:
                data['Value'] = {
                    '$type': asi_type_label,
                    '$value': value
                }
            return data
    raise ValueError('value arg is not of a supported type: str, float, int, bool')


def generic_property_data_collection_to_dict(collection):
    if collection['$type'] != ASI_GENERIC_PROPERTY_DATA_COLLECTION:
        msg = 'Input must be: {0}'.format(ASI_GENERIC_PROPERTY_DATA_COLLECTION)
        raise ValueError(msg)
    output = {}
    for prop in collection['$values']:
        k = prop['Name']
        try:
            output[k] = prop['Value']['$value']
        except TypeError:
            output[k] = prop['Value']
        except KeyError:
            output[k] = None
    return output


def dict_to_generic_property_data_collection(d):
    return {
        '$type': ASI_GENERIC_PROPERTY_DATA_COLLECTION,
        '$values': [generic_property_data(k, v) for k, v in d.items()]
    }



def snake_case(string):
    out = []
    for part in string.split():
        part_chars = []
        for index, s in enumerate(part):
            if index == 0:
                part_chars.append(s.lower())
            else:
                if s.isupper():
                    if string[index - 1].islower():
                        part_chars.append('_')
                part_chars.append(s.lower())
        out.append(''.join(part_chars))
    return '_'.join(out)



import requests


class UserExistsError(Exception):
    pass


class ImisError(Exception):
    pass


def create(client, username, imis_id):
    headers = {
        'Authorization': client.auth.authorization_header
    }
    data = {
        '$type': 'Asi.Soa.Membership.DataContracts.UserData, Asi.Contracts',
        'UserName': username,
        'UserId': imis_id
    }
    url = client.url + '/api/User'
    response = requests.post(url, json=data, headers=headers)
    try:
        response.raise_for_status()
    except:
        error_response_body = response.content.decode()
        if 'DuplicateProviderUserKey' in error_response_body:
            raise UserExistsError(f'{username} exists')
        raise ImisError(error_response_body)



import requests

from imis.utils import dict_to_generic_property_data_collection, generic_property_data_collection_to_dict


def create(client, data):
    headers = {
        'Authorization': client.auth.authorization_header
    }
    body = {
        '$type': 'Asi.Soa.Core.DataContracts.GenericEntityData, Asi.Contracts',
        'PrimaryParentEntityTypeName': 'Party',
        'Properties': dict_to_generic_property_data_collection(data)
    }
    url = client.url + '/api/Activity'
    response = requests.post(url, headers=headers, json=body)
    response.raise_for_status()
    return generic_property_data_collection_to_dict(response.json()['Properties'])



import requests


def _get_name_fin_data(client, imis_id):
    headers = {
        'Authorization': client.auth.authorization_header
    }
    url = client.url + f'/api/CsNameFin/{imis_id}'
    response = requests.get(url, headers=headers)
    response.raise_for_status()
    return response.json()


def _update_name_fin_data(client, imis_id, payload):
    headers = {
        'Authorization': client.auth.authorization_header
    }
    url = client.url + f'/api/CsNameFin/{imis_id}'
    response = requests.put(url, json=payload, headers=headers)
    response.raise_for_status()
    return response.json()


def update_renewed_thru(client, imis_id, renewed_thru):
    if renewed_thru is not None:
        renewed_thru = renewed_thru.isoformat()
    headers = {
        'Authorization': client.auth.authorization_header
    }
    name_fin_data = _get_name_fin_data(client, imis_id)
    for index, element in enumerate(name_fin_data['Properties']['$values']):
        if element['Name'] == 'RENEWED_THRU':
            name_fin_data['Properties']['$values'][index]['Value'] = renewed_thru
    _update_name_fin_data(client, imis_id, name_fin_data)



from imis.utils import snake_case


class BasicRecord:

    def __init__(self, **kwargs):
        self.__dict__ = {snake_case(k): v for k, v in  kwargs.items()}

    def __getattr__(self, name):
        try:
            return self.__dict__[name.lower()]
        except KeyError:
            raise AttributeError(f'{name} does not exist on this object')

    def __setattr__(self, name, value):
        super().__setattr__(snake_case(name), value)

    def __repr__(self):
        return '<BasicRecord>'

    def __str__(self):
        return repr(self)



import requests

from imis.utils import generic_property_data_collection_to_dict


def _raw_iqa_results(client, query_name, offset=0, *parameters):
    headers = {'Authorization': client.auth.authorization_header}
    url = client.url + f'/api/iqa?QueryName={query_name}&offset={offset}'
    for param in parameters:
        url += f'&parameter={param}'

    response = requests.get(url, headers=headers)
    response.raise_for_status()
    data = response.json()
    yield data
    if data['HasNext']:
        yield from _raw_iqa_results(client, query_name, data['NextOffset'], *parameters)


def iter_items(client, query_name, offset=0, *parameters):
    for page in _raw_iqa_results(client, query_name, *parameters):
        for item in page['Items']['$values']:
            yield generic_property_data_collection_to_dict(item['Properties'])



import setuptools

setuptools.setup(
    name="imis-projectweekend", # Replace with your own username
    version="0.1.1",
    author="Brian Hines",
    author_email="brian@projectweekend.net",
    description="A Python client for the iMIS REST API",
    long_description="A Python client for the iMIS REST API",
    url="https://github.com/projectweekend/imis",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests'
    ],
    python_requires='>=3.6',
)



import requests

import imis.activity as activity
import imis.iqa as iqa
import imis.name_fin as name_fin
import imis.record as record
import imis.user as user



class ApiError(Exception):
    pass


class AuthError(ApiError):
    pass


class Auth:

    def __init__(self, access_token, token_type, expires_in, user_name):
        self.access_token = access_token
        self.token_type = token_type
        self.expires_in = expires_in
        self.user_name = user_name

    @property
    def authorization_header(self):
        return '{0} {1}'.format(self.token_type, self.access_token)

    @classmethod
    def authenticate(cls, url, username, password):
        data = {
            'grant_type': 'password',
            'username': username,
            'password': password
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
        response = requests.post(url=url, data=data, headers=headers)
        if response.status_code == 400:
            data = response.json()
            raise AuthError(data['error_description'])

        data = response.json()
        return cls(access_token=data['access_token'],
                    token_type=data['token_type'],
                    expires_in=data['expires_in'],
                    user_name=data['userName'])


class Client:

    _authenticator = Auth

    def __init__(self, url, username, password):
        self.url = url
        self.username = username
        self.password = password
        self._auth = Client._authenticator.authenticate(f'{url}/token', username, password)

    @property
    def auth(self):
        if self._auth is None:
            self._auth = Client._authenticator.authenticate(f'{self.url}/token', self.username, self.password)
        return self._auth

    @auth.setter
    def auth(self, val):
        self._auth = val

    def iqa(self, query_name, *parameters):
        for item in iqa.iter_items(self, query_name, *parameters):
            yield record.BasicRecord(**item)

    def create_activity(self, activity_dict):
        return activity.create(self, activity_dict)

    def create_user(self, username, imis_id):
        return user.create(self, username, imis_id)

    def update_renewed_thru(self, imis_id, renewed_thru):
        name_fin.update_renewed_thru(self, imis_id, renewed_thru)
